from crawliexpress.exceptions import (
    CrawliexpressException,
    CrawliexpressCaptchaException,
)
from crawliexpress.client import Client
from crawliexpress.item import Item
from crawliexpress.feedback_page import FeedbackPage
from crawliexpress.feedback import Feedback
from crawliexpress.search_page import SearchPage



from crawliexpress.exception import CrawliexpressException
from crawliexpress.feedback import Feedback

from bs4 import BeautifulSoup
import re


class FeedbackPage:
    feedbacks = None
    page = None
    known_pages = None

    def from_html(self, html):
        soup = BeautifulSoup(html, "lxml")

        # feedbacks
        feedbacks = self.feedbacks = list()
        for node in soup.find_all("", class_="feedback-item"):
            feedback = Feedback()
            feedback.from_node(node)
            feedbacks.append(feedback)

        # page
        node_pages = soup.find("", class_="ui-pagination-navi")
        if node_pages is None:
            raise CrawliexpressException("could not find pagination node")

        node_page = node_pages.find("", class_="ui-pagination-active")
        if node_page is None:
            raise CrawliexpressException("could not find current page")
        page = node_page.text
        if re.match(r"^\d+$", page) is None:
            raise CrawliexpressException("current page is not a number")
        self.page = int(page)

        # pages
        known_pages = self.known_pages = list()
        for node_page in node_pages.find_all("", class_="ui-goto-page"):
            page = node_page.text
            if re.match(r"^\d+$", page) is not None:
                known_pages.append(int(page))

    def has_next_page(self):
        has = False
        for page in self.known_pages:
            if page >= self.page:
                has = True
                break

        return has

    def __iter__(self):
        yield "feedbacks", [dict(feedback) for feedback in self.feedbacks]
        yield "page", self.page
        yield "known_pages", self.known_pages
        yield "has_next_page", self.has_next_page()



from crawliexpress.exception import CrawliexpressException

import re
import json
import _jsonnet


RUN_PARAMS_RE = re.compile(r"window\.runParams = ({.+?});$", re.MULTILINE | re.DOTALL)


class Item:
    run_params = None
    product_id = None
    owner_member_id = None
    company_id = None

    def from_html(self, html):

        matches = RUN_PARAMS_RE.search(html)
        if matches is None:
            raise CrawliexpressException("could not find runParams")

        self.run_params = run_params = json.loads(
            _jsonnet.evaluate_snippet("snippet", matches.group(1))
        )
        if "data" not in run_params or "feedbackModule" not in run_params["data"]:
            raise CrawliexpressException("unknown runParams structure")

        feedback_module = run_params["data"]["feedbackModule"]

        self.product_id = feedback_module["productId"]
        self.owner_member_id = feedback_module["sellerAdminSeq"]
        self.company_id = feedback_module["companyId"]



from crawliexpress.exception import CrawliexpressException

import re
from bs4 import BeautifulSoup


RATING_RE = re.compile(r"width:(\d+)%")


class Feedback:
    user = None
    profile = None
    country = None
    rating = None
    datetime = None
    comment = None
    images = None

    def from_node(self, node):

        # user name
        node_user = node.find("", class_="user-name")
        if node_user is None:
            raise CrawliexpressException("could not find feedback user node")
        self.user = node_user.text.strip()

        # profile
        node_profile = node_user.find("a")
        # profile can be hidden
        if node_profile is not None:
            self.profile = node_profile["href"]

        # country
        node_country = node.find("", class_="user-country")
        if node_country is None:
            raise CrawliexpressException("could not find feedback country node")
        self.country = node_country.text

        # rating
        node_rating = node.find("", class_="star-view")
        if node_rating is None:
            raise CrawliexpressException("could not find feedback rating node")
        for node_rating_stars in node_rating.children:
            rating_matches = RATING_RE.search(node_rating_stars["style"])
            if rating_matches is not None:
                self.rating = int(rating_matches.group(1))

        # review part
        node_review = node.find("", class_="buyer-review")
        if node_review is None:
            raise CrawliexpressException("could not find feedback review node")

        node_comment = node_review.find("", class_="buyer-feedback")
        if node_comment is None:
            raise CrawliexpressException("could not find feedback comment node")

        # datetime
        node_datetime = node_comment.find("", class_="r-time-new")
        if node_datetime is None:
            raise CrawliexpressException("could not find feedback datetime")
        self.datetime = node_datetime.text

        # comment
        node_comment_span = node_comment.find("span")
        if node_comment_span is None:
            raise CrawliexpressException("could not find feedback comment")
        self.comment = node_comment_span.text

        # images
        images = self.images = list()
        node_images = node.find("", class_="r-photo-list")
        if node_images is not None:
            for node_image in node_images.find_all("img"):
                images.append(node_image["src"])

    def __iter__(self):
        yield "user", self.user
        yield "profile", self.profile
        yield "country", self.country
        yield "rating", self.rating
        yield "datetime", self.datetime
        yield "comment", self.comment
        yield "images", self.images



from crawliexpress.exception import CrawliexpressException


class SearchPage:
    page = None
    result_count = None
    size_per_page = None
    items = None

    def from_json(self, page, json):
        self.page = page
        self.result_count = json["resultCount"]
        self.size_per_page = json["resultSizePerPage"]
        self.items = json["items"]

    def has_next_page(self):
        return len(self.items) == self.size_per_page

    def __iter__(self):
        yield "page", self.page
        yield "result_count", self.result_count
        yield "size_per_page", self.size_per_page
        yield "items", self.items
        yield "has_next_page", self.has_next_page()



from crawliexpress.exception import (
    CrawliexpressException,
    CrawliexpressCaptchaException,
)
from crawliexpress.item import Item
from crawliexpress.feedback_page import FeedbackPage
from crawliexpress.search_page import SearchPage

import requests
import urllib


FEEDBACK_URL = "https://feedback.aliexpress.com/display/productEvaluation.htm"


class Client:
    base_url = None

    # cookies to avoid capcha
    xman_t = None
    x5sec = None

    # cookies for category search
    aep_usuc_f = None

    def __init__(self, base_url, xman_t=None, x5sec=None, aep_usuc_f=None):
        self.base_url = base_url
        self.xman_t = xman_t
        self.x5sec = x5sec
        self.aep_usuc_f = aep_usuc_f

    def __analyze_response(self, response):
        if response.status_code != 200:
            raise CrawliexpressException(f"invalid status code {response.status_code}")
        elif (
            not response.headers["Content-Type"].startswith("application/json")
            and "captcha" in response.text
        ):
            raise CrawliexpressCaptchaException()

    def get_item(self, item_id):
        r = requests.get(f"{self.base_url}/item/{item_id}.html")
        self.__analyze_response(r)
        item = Item()
        item.from_html(r.text)
        return item

    def get_feedbacks(
        self,
        product_id,
        owner_member_id,
        company_id=None,
        v=2,
        member_type="seller",
        page=1,
        with_picture=False,
    ):
        params = urllib.parse.urlencode(
            {
                "productId": product_id,
                "ownerMemberId": owner_member_id,
                "companyId": company_id,
                "v": v,
                "memberType": member_type,
                "page": str(page),
                "withPictures": with_picture,
            }
        )
        url = f"{FEEDBACK_URL}?{params}"
        r = requests.get(url)
        self.__analyze_response(r)
        feedback_page = FeedbackPage()
        feedback_page.from_html(r.text)
        return feedback_page

    def get_search(self, page_no, category_id=0, search_text=None, sort_by="default"):

        """
        sort_by can be:
        - default: best
        - orders: total_tranpro_desc
        """

        params = {
            "trafficChannel": "main",
            "d": "y",
            "CatId": category_id,
            "ltype": "wholesale",
            "SortType": sort_by,
            "page": page_no,
            "origin": "y",
            "isrefine": "y",
        }
        if search_text is not None:
            params["SearchText"] = search_text
        params = urllib.parse.urlencode(params)
        url = f"{self.base_url}/glosearch/api/product?{params}"
        headers = {"Accept": "application/json"}
        cookies = {
            "xman_t": self.xman_t,
            "x5sec": self.x5sec,
            "aep_usuc_f": self.aep_usuc_f,
        }
        r = requests.get(url, headers=headers, cookies=cookies)
        self.__analyze_response(r)
        search_page = SearchPage()
        search_page.from_json(page_no, r.json())
        return search_page



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="crawliexpress",  # Replace with your own username
    version="0.1.0",
    author="ToucanTocard",
    author_email="contact@robin.ninja",
    description="Python3 library to ease Aliexpress crawling",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/toucantocard/crawliexpress",
    packages=setuptools.find_packages(include=["crawliexpress"]),
    classifiers=[
        "Development Status :: 4 - Beta",  # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    license="MIT",
    python_requires=">=3.6",
    install_requires=["requests", "jsonnet", "bs4", "lxml"],
    setup_requires=[],
    tests_require=[],
    keywords=["aliexpress"],
)
